/* knapsack.c */

/* Direttive */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "knapsack.h"


/* Prototipi delle procedure secondarie */

void InterpretaLineaComando (int argc, char *argv[], char *filedati);

/* Risolve l'istanza I del problema dello zaino con l'euristica greedy base.
   Nota: assume che la soluzione S sia inizialmente vuota */
void GreedyBanaleKP (datiKP *pI, soluzioneKP *pS);


/* Programma principale */
int main (int argc, char *argv[])
{
  /* Parte dichiarativa */
  char filedati[ROW_LENGTH];
  datiKP I;
  soluzioneKP S;
  clock_t inizio, fine;
  double tempo;


  /* Parte esecutiva */
  InterpretaLineaComando(argc,argv,filedati);

  /* Carica i dati */
  CaricaDatiKP(filedati,&I);

  /* Crea una soluzione vuota */
  CreaSoluzioneKP(I.n,&S);

  /* Risolve il problema dello zaino */
  inizio = clock();
  GreedyBanaleKP(&I,&S);
  fine = clock();
  tempo = ((double) (fine - inizio))/CLOCKS_PER_SEC;

  /* Stampa il tempo di calcolo e la soluzione */
  printf("CPU = %f\n",tempo);
  StampaSoluzioneKP(I.n,&S);

  /* Dealloca le strutture dati dinamiche */
  DistruggeDatiKP(&I);
  DistruggeSoluzioneKP(&S);

  return EXIT_SUCCESS;
}


/* Definizione delle procedure secondarie */

void InterpretaLineaComando (int argc, char *argv[], char *filedati)
{
  if (argc != 2)
  {
    fprintf(stderr,"Errore nella linea di comando!\n");
    fprintf(stderr,"%s [filedati]\n",argv[0]);
    exit(EXIT_FAILURE);
  }

  strcpy(filedati,argv[1]);
}


/* Risolve l'istanza I del problema dello zaino con l'euristica greedy base.
   Nota: assume che la soluzione S sia inizialmente vuota */
void GreedyBanaleKP (datiKP *pI, soluzioneKP *pS)
{
}
