/* knapsack.c */

/* Direttive */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "alloc.h"
#include "knapsack.h"


/* Prototipi delle procedure secondarie */

void InterpretaLineaComando (int argc, char *argv[], char *filedati);

/* Risolve l'istanza I del problema dello zaino con l'euristica greedy.
   Nota: assume che la soluzione S sia inizialmente vuota */
void GreedyKP (datiKP *pI, soluzioneKP *pS);

void creaheapindiretto (vint Indice, vdouble V, int n);

void aggiornaheapindiretto (vint Indice, vdouble V, int n, int i);

void Scambia (int *pa, int *pb);


/* Programma principale */
int main (int argc, char *argv[])
{
  /* Parte dichiarativa */
  char filedati[ROW_LENGTH];
  datiKP I;
  soluzioneKP S;
  clock_t inizio, fine;
  double tempo;


  /* Parte esecutiva */
  InterpretaLineaComando(argc,argv,filedati);

  /* Carica i dati */
  CaricaDatiKP(filedati,&I);

  /* Crea una soluzione vuota */
  CreaSoluzioneKP(I.n,&S);

  /* Risolve il problema dello zaino */
  inizio = clock();
  GreedyKP(&I,&S);
  fine = clock();
  tempo = ((double) (fine - inizio))/CLOCKS_PER_SEC;

  /* Stampa il tempo di calcolo e la soluzione */
  printf("CPU = %f\n",tempo);
  StampaSoluzioneKP(I.n,&S);

  /* Dealloca le strutture dati dinamiche */
  DistruggeDatiKP(&I);
  DistruggeSoluzioneKP(&S);

  return EXIT_SUCCESS;
}


/* Definizione delle procedure secondarie */

void InterpretaLineaComando (int argc, char *argv[], char *filedati)
{
  if (argc != 2)
  {
    fprintf(stderr,"Errore nella linea di comando!\n");
    fprintf(stderr,"%s [filedati]\n",argv[0]);
    exit(EXIT_FAILURE);
  }

  strcpy(filedati,argv[1]);
}


void creaheapindiretto (vint Indice, vdouble V, int n)
{
  int i;

  for (i = n/2; i >= 1; i--)
    aggiornaheapindiretto(Indice,V,n,i);
}


void aggiornaheapindiretto (vint Indice, vdouble V, int n, int i)
{
  int s, d;
  int iMax;

  s = 2*i;
  d = 2*i+1;

  iMax = i;
  if ( (s <= n) && (V[Indice[s]] > V[Indice[iMax]]) ) iMax = s;
  if ( (d <= n) && (V[Indice[d]] > V[Indice[iMax]]) ) iMax = d;

  if (iMax != i)
  {
    Scambia(&Indice[iMax],&Indice[i]);
    aggiornaheapindiretto(Indice,V,n,iMax);
  }
}


void Scambia (int *pa, int *pb)
{
  int temp;

  temp = *pa;
  *pa = *pb;
  *pb = temp;
}


/* Risolve l'istanza I del problema dello zaino con l'euristica greedy.
   Nota: assume che la soluzione S sia inizialmente vuota */
void GreedyKP (datiKP *pI, soluzioneKP *pS)
{
  vint Indice;   /* vettore degli indici degli elementi */
  vdouble phi_v; /* vettore dei valori unitari degli elementi */
  int nEst;      /* numero degli elementi ancora candidati a entrare nello zaino */
  int i, iMax;
  int cr;        /* capacita' residua dello zaino */


  /* Crea un vettore di indici gli elementi non ancora considerati (B' = B) */
  Indice = intalloc(pI->n+1);
  for (i = 1; i <= pI->n; i++)
    Indice[i] = i;

  /* Crea un vettore di rapporti valore/volume (phi/v) */
  phi_v = doublealloc(pI->n+1);
  for (i = 1; i <= pI->n; i++)
    phi_v[i] = (double) pI->phi[i]/pI->v[i];

  /* Trasforma il vettore di indici in un heap indiretto */
  creaheapindiretto(Indice,phi_v,pI->n);

  /* Finche' ci sono elementi ancora non considerati (B' non vuoto) */
  nEst = pI->n;
  cr = pI->V;
  while ( (nEst > 0) && (cr > 0) )
  {
    /* Trova l'elemento non considerato di valore massimo */
    iMax = Indice[1];

    /* Se tale elemento sta nello zaino, lo aggiunge alla soluzione */
    if (pI->v[iMax] <= cr)
    {
      pS->x[iMax] = TRUE;
      pS->f = pS->f + pI->phi[iMax];
      cr -= pI->v[iMax];
    }

    /* Cancella l'elemento dall'heap indiretto B' */
    Indice[1] = Indice[nEst];
    nEst--;
    aggiornaheapindiretto(Indice,phi_v,nEst,1);
  }

  /* Distrugge l'heap indiretto B' e il vettore phi/v */
  intfree(&Indice);
  doublefree(&phi_v);
}
